/**
 * Represent the state of each character in the string.
 * `0` represents incomplete, `1` represents correct and, `2` represents incorrect.
 */
declare type CharStateType = 0 | 1 | 2;
interface TypingOptionsType {
    /**
     * Move on to the next word when space is inputted, defaults to `true`.
     */
    skipCurrentWordOnSpace: boolean;
    /**
     * Stay on the current index when the inputted character is wrong, defaults to `false`.
     */
    pauseOnError: boolean;
}
/**
 * Properties of the typing game hook.
 */
interface TypingStateType extends TypingOptionsType {
    /**
     * The inputted string to be used.
     */
    chars: string;
    /**
     * Array of each character's state in the string.
     */
    charsState: CharStateType[];
    /**
     * Length of the string used.
     */
    length: number;
    /**
     * Current index of the character the user have typed till.
     */
    currIndex: number;
    /**
     * Current character the user have typed till.
     */
    currChar: string;
    /**
     * Number of correct character the user had typed.
     */
    correctChar: number;
    /**
     * Number of incorrect character the user had typed.
     */
    errorChar: number;
    /**
     * Represent the current state.
     * `0` typing haven't started, `1` typing started, `2` typing ended.
     */
    phase: 0 | 1 | 2;
    /**
     * Time in milliseconds when the typing started
     */
    startTime: number | null;
    /**
     * Time in milliseconds when the typing ended
     */
    endTime: number | null;
}
/**
 * Methods of the typing game hook.
 */
declare type TypingActionType = {
    /**
     * Reset the typing sequence
     */
    resetTyping: () => void;
    /**
     * Ends the typing sequence but not reset it.
     */
    endTyping: () => void;
    /**
     * Insert a character into the current typing sequence.
     * @param {string | null} char A character to be inserted.
     * If falsy or no argument is supplied, skip the current character.
     */
    insertTyping: (char?: string | null) => void;
    /**
     * Delete a character from the current typing sequence.
     * @param {boolean} [deleteWord] If `true`, deletes the whole of the current word. Defaults to `false`.
     */
    deleteTyping: (deleteWord?: boolean) => void;
    /**
     * Set the current index manually.
     * @param {number} num Allows from -1 to length - 1 of the text, numbers that falls outside of the range will return a false
     */
    setCurrIndex: (num: number) => boolean;
};
/**
 * React hook to create typing game/practice/test.
 * @param {string} text A string of words to be used for the typing sequence.
 * @param {Object} [options] Addition options to customize the functionality of the typing sequence.
 * @param {boolean} [options.skipCurrentWordOnSpace] Move on to the next word when space is inputted, defaults to `true`
 * @param {boolean} [options.pauseOnError] Stay on the current index when the inputted character is wrong, defaults to `false`
 * @returns Returns the state and the actions available for the typing hook
 */
declare const useTypingGame: (text: string, options?: Partial<TypingOptionsType>) => {
    states: TypingStateType;
    actions: TypingActionType;
};
export default useTypingGame;
